#[
  2019/01 by audin (http://mpu.seesaa.net)
  Nim language test program for Arduino UNO and its compatilbes.
  AVR = atmega328p / 16MHz
  avr-gcc: ver.5.4.0
  nim: 0.19.2
  UART baudrate 9600bps
]#

import iom328p

{.compile:"xprintf.c".}
proc xprintf(formatstr: cstring){.header: "../xprintf.h", importc: "xprintf", varargs,used.}
###
proc delay_ms(ms:cdouble){.header:"<util/delay.h>",importc:"_delay_ms",used.}
proc UART_putc(c: char) # forward definition

# Use ChaN's xprintf()
let xfunc_out{.exportc.} = UART_putc

# UART setting
const BAUD_9600bps = 103
template TX_DATA():untyped = UDR0

proc initUart( baudFactor :int ) =
    UBRR0H.st8 (baudFactor shr 8 ).uint8
    UBRR0L.st8  baudFactor.uint8
    UCSR0B.st8 ((1 shl RXEN0) or (1 shl TXEN0)).uint8 # enable TX,RX
    UCSR0C.st8 ((3 shl UCSZ00) or (0 shl USBS0) or (0 shl UPM00)).uint8

template isUART0_TxRx_FIFO_empty():bool =
    (UCSR0A.ld8 and (1 shl UDRE0)) != 0

proc UART_putc(c: char) =
    while not isUART0_TxRx_FIFO_empty():
        discard
    TX_DATA.st8 c.uint8

# LED setting
proc led_setup()=
    DDRB.st8 ( DDRB.ld8 or BV(DDB5) )

proc led_on()=
    PORTB.st8 ( PORTB.ld8 or BV(PORTB5) )

proc led_off()=
    PORTB.st8  (PORTB.ld8.int and not BV(PORTB5).int).uint8

proc wait_ms(ms:int)=
    var i {.volatile.}:int = ms
    while i > 0:
        dec(i)
        delay_ms(1)

# main program
var num = 0
led_setup()
initUart(BAUD_9600bps)
while true:
    led_on()
    wait_ms(500)
    led_off()
    wait_ms(500)
    xprintf("\n Number = %d", num)
    num += 1

